#ifdef HAVE_CONFIG_H
    #include "autoconfig.h"
#endif

#include <sys/stat.h>
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include <libconfig.h>
#include <curl/curl.h>

#include "dsa_network_handler.h"

#define DSA_USER_CONFIG "/etc/dsa/dsa_user.cfg"
#define NETWORK_URL "https://testdsservices.aizo.com/internal/dss/v1_0/DSSApartment/usbUpdateChecker"
#define NETWORK_ID "3e78d368383d12cd41c253279ee44918296228496dceb81c30df6684e8d1db68"
#define LOCATION_FILE "/media/dss11usb/dss11-usb-upgrade/update-info.txt"
#define ETHERNET_FILE "/sys/class/net/eth0/address"

static int network_notification()
{
    char location[1000] = { 0, };
    char eth0[100] = { 0, };
    char *parameters;

    FILE *locationFile = fopen(LOCATION_FILE, "r");
    if (locationFile != NULL)
    {
        char *p = fgets(location, 1000, locationFile);
        if (p && p[strlen(p) - 1] == '\n')
        {
            p[strlen(p) - 1] = 0;
        }
        fclose(locationFile);
    }

    FILE *ethernetFile = fopen(ETHERNET_FILE, "r");
    if (ethernetFile != NULL)
    {
        char *p = fgets(eth0, 100, ethernetFile);
        if (p && p[strlen(p) - 1] == '\n')
        {
            p[strlen(p) - 1] = 0;
        }
        fclose(ethernetFile);
    }

    parameters = malloc(2000);
    if (parameters == NULL)
    {
        fprintf(stderr, "Error allocating memory for network notification\n");
        return EXIT_FAILURE;
    }

#define DO_POST
#ifdef DO_POST
    strcpy(parameters, NETWORK_URL);
    strcat(parameters, "?token=");
    strcat(parameters, NETWORK_ID);
    strcat(parameters, "&mac=");
    strcat(parameters, eth0);

    struct memory_struct data;
    int retries;
    int nres;
    for (retries = 0; retries < 5; retries++)
    {
        data.memory = malloc(1);
        data.size = 0;
        nres = network_post(parameters, location, &data);
        free(data.memory);

        if (nres != 0)
        {
            fprintf(stderr, "Network communication failure [%d]\n", nres);
        }
        else
        {
            if (data.http_code / 100 == 2)
            {
                fprintf(stderr, "Network notification success [%d]\n", data.http_code);
                break;
            }
            else
            {
                fprintf(stderr, "Network notification failure [%d]:\n Service %s Parameter %s\n",
                        data.http_code, NETWORK_URL, parameters);
            }
        }
        sleep(60);
    }
#else
    strcpy(parameters, "?token=");
    strcat(parameters, NETWORK_ID);
    strcat(parameters, "&location=");
    strcat(parameters, location);
    strcat(parameters, "&mac=");
    strcat(parameters, eth0);

    struct memory_struct data;
    int retries;
    int nres;
    for (retries = 0; retries < 5; retries++)
    {
        data.memory = malloc(1);
        data.size = 0;
        nres = network_query(NETWORK_URL, parameters, &data);
        free(data.memory);

        if (nres != 0)
        {
            fprintf(stderr, "Network communication failure [%d]\n", nres);
        }
        else
        {
            if (data.http_code / 100 == 2)
            {
                fprintf(stderr, "Network notification success [%d]\n", data.http_code);
                break;
            }
            else
            {
                fprintf(stderr, "Network notification failure [%d]:\n Service %s Parameter %s\n",
                        data.http_code, NETWORK_URL, parameters);
            }
        }
        sleep(60);
    }
#endif

    free(parameters);
    return nres;
}

static int write_config(char *file)
{
    config_t config;
    config_setting_t* parent;
    config_setting_t* setting;
    config_setting_t* group;
    int ret = EXIT_SUCCESS;

    config_init(&config);
    if (!config_read_file(&config, DSA_USER_CONFIG))
    {
        fprintf(stderr, "Error in " DSA_USER_CONFIG ": %d %s\n",
                        config_error_line(&config),
                        config_error_text(&config));
        ret = EXIT_FAILURE;
        goto out;
    }

    parent = config_root_setting(&config);

    group = config_setting_get_member(parent, "digitalstrom");
    if (group == NULL)
    {
        group = config_setting_add(parent, "digitalstrom", CONFIG_TYPE_GROUP);
    }

    if (group == NULL)
    {
        fprintf(stderr, "Error: unexpected configuration type\n");
        ret = EXIT_FAILURE;
        goto out;
    }

    parent = group;

    group = config_setting_get_member(parent, "tasks");
    if (group == NULL)
    {
        group = config_setting_add(parent, "tasks", CONFIG_TYPE_GROUP);
    }

    if (group == NULL)
    {
        fprintf(stderr, "Error: unexpected configuration type\n");
        ret = EXIT_FAILURE;
        goto out;
    }

    parent = group;

    group = config_setting_get_member(parent, "device-pulse");
    if (group == NULL)
    {
        group = config_setting_add(parent, "device-pulse", CONFIG_TYPE_GROUP);
    }

    if (group == NULL)
    {
        fprintf(stderr, "Error: unexpected configuration type\n");
        ret = EXIT_FAILURE;
        goto out;
    }

    setting = config_setting_get_member(group, "active");
    if (setting == NULL)
    {
        setting = config_setting_add(group, "active", CONFIG_TYPE_BOOL);
    }

    if (setting == NULL)
    {
        fprintf(stderr, "Error: unexpected configuration type\n");
        ret = EXIT_FAILURE;
        goto out;
    }

    config_setting_set_bool(setting, true);

    setting = config_setting_get_member(group, "last-run");
    if (setting == NULL)
    {
        setting = config_setting_add(group, "last-run", CONFIG_TYPE_STRING);
    }

    if (setting == NULL)
    {
        fprintf(stderr, "Error: unexpected configuration type\n");
        ret = EXIT_FAILURE;
        goto out;
    }

    config_setting_set_string(setting, "0");

    if (!config_write_file(&config, DSA_USER_CONFIG))
    {
        fprintf(stderr, "dSA configuration tuner: error writing dSA pulse "
                        "task configuration\n");
        ret = EXIT_FAILURE;
    }
    else
    {
        printf("dSA configuration tuner: pulse task configuration written\n");
    }

out:
    config_destroy(&config);
    return ret;
}

int main()
{
    struct stat statbuf;
    int rc;

    if (stat(DSA_USER_CONFIG, &statbuf) != 0)
    {
        FILE *f = fopen(DSA_USER_CONFIG, "wb");
        if (f == NULL)
        {
            fprintf(stderr, "Could not create file %s: %s\n",
                    DSA_USER_CONFIG, strerror(errno));
            return EXIT_FAILURE;
        }
        fclose(f);
    }
    else if (!S_ISREG(statbuf.st_mode))
    {
        fprintf(stderr, "Error: " DSA_USER_CONFIG " is not a regular file!\n");
        return EXIT_FAILURE;
    }

    rc = write_config(DSA_USER_CONFIG);
    if (rc != EXIT_SUCCESS) {
        fprintf(stderr, "Error: could not write to dsa config file [%d]!\n", rc);
    }

    curl_global_init(CURL_GLOBAL_ALL);
    rc = network_notification();
    if (rc != EXIT_SUCCESS) {
        fprintf(stderr, "Error: could not send notification [%d]!\n", rc);
    }

    return 0;
}
